=== modified file 'NEWS'
--- NEWS	2009-05-17 11:10:38 +0000
+++ NEWS	2009-06-17 22:33:55 +0000
@@ -4,6 +4,8 @@
 dev [future]
 ---------------
 
+    - Add bug links in revision informations (Alexandre Garnier, #314052)
+
     - Make sure that binary files aren't annotated. (Martin Albisetti,
       #258848)
 
@@ -115,6 +117,23 @@
 
     - Fix 'bzr serve --http' errors. (Matt Nordhoff, #377551)
 
+    - Added the option to hide branches by setting http_serve = False
+      in locations.conf (Martin Albisetti)
+
+    - Fix serving branches over HTTP. (Matt Nordhoff, Jelmer Vernooij,
+      #380026)
+
+    - Install loggerhead as a bzr plugin by default (Jelmer Vernooij)
+
+    - Fix logging 404 Not Found responses (Matt Nordhoff, #381029)
+
+    - Bumped minimunm bzrlib version to 1.13 (Martin Albisetti)
+
+    - Make sure the Atom feeds (nearly) validate. (Matt Nordhoff, #247162)
+
+    - Support serving branches over HTTP using the smart server protocol. 
+      (Jelmer Vernooij, #306853)
+
 1.10 [22Dec2008]
 ---------------
 

=== renamed file 'README.txt' => 'README'
--- README.txt	2009-03-17 22:31:42 +0000
+++ README	2009-05-27 12:52:49 +0000
@@ -1,7 +1,7 @@
 LOGGERHEAD
 ==========
 
-[ Version 1.6 for Bazaar 1.6 ]
+[ Version 1.10 for Bazaar 1.6 ]
 
 Loggerhead is a web viewer for Bazaar branches.  It can be used to
 navigate a branch history, annotate files, perform searches... all the
@@ -63,31 +63,14 @@
 USING A CONFIG FILE
 -------------------
 
-Previous versions of Loggerhead read their configuration from a config
-file.  This mode of operation is still supported by the
-'start-loggerhead' script.  A 'loggerhead.conf.example' file is
-included in the source which has comments explaining the various
-options.
-
-Loggerhead can then be started by running::
-
-    $ ./start-loggerhead
-
-This will run loggerhead in the background, listening on port 8080 by
-default.
-
-To stop Loggerhead, run::
-
-    $ ./stop-loggerhead
-
-In the configuration file you can configure projects, and branches per
-project.  The idea is that you could be publishing several (possibly
-unrelated) projects through the same loggerhead instance, and several
-branches for the same project.  See the "loggerhead.conf.example" file
-included with the source.
-
-A debug and access log are stored in the logs/ folder, relative to
-the location of the start-loggerhead script.
+To hide branches from being displayed, add to ``~/.bazaar/locations.conf``,
+under the branch's section:
+
+    [/path/to/branch]
+    http_serve = False
+
+
+More configuration options to come soon.
 
 
 SERVING LOGGERHEAD FROM BEHIND APACHE
@@ -107,13 +90,14 @@
 run behind a proxy at the root of a site, but if you're running it at
 some path into the site, you'll need to specify is using '--prefix=/some_path'.
 
-FILES CHANGED CACHE
--------------------
-
-To speed up the display of the changelog view for large trees,
-loggerhead can be configured to cache the files changes between
-revisions.  Set the 'cachepath' value in the config file.
-
+
+SEARCH
+------
+
+Search is currently supported by using the bzr-search plugin (available
+at: ``https://launchpad.net/bzr-search``
+You need to have the plugin installed and each branch indexed to allow
+searching on branches.
 
 SUPPORT
 -------

=== modified file '__init__.py'
--- __init__.py	2009-05-17 11:10:38 +0000
+++ __init__.py	2009-06-18 00:26:05 +0000
@@ -36,59 +36,83 @@
     import bzrlib
     from bzrlib.api import require_any_api
 
-    require_any_api(bzrlib, [(1, 11, 0), (1, 13, 0), (1, 15, 0)])
-
-    # TODO: This should provide a new type of server that can be used by bzr
-    # serve, maybe through a registry, rather than overriding the command.  Though
-    # maybe we should keep the wrapper to work with older bzr releases, at least
-    # for a bit.
+    require_any_api(bzrlib, [(1, 13, 0), (1, 15, 0), (1, 16, 0), (1, 17, 0)])
 
     # NB: Normally plugins should lazily load almost everything, but this
     # seems reasonable to have in-line here: bzrlib.commands and options are
     # normally loaded, and the rest of loggerhead won't be loaded until serve
     # --http is run.
-        
-    import bzrlib.builtins
-    from bzrlib.commands import get_cmd_object, register_command
-    from bzrlib.option import Option
-
-    _original_command = get_cmd_object('serve')
-
+
+    # transport_server_registry was added in bzr 1.16. When we drop support for
+    # older releases, we can remove the code to override cmd_serve.
+
+    try:
+        from bzrlib.transport import transport_server_registry
+    except ImportError:
+        transport_server_registry = None
+
+    DEFAULT_HOST = '0.0.0.0'
     DEFAULT_PORT = 8080
-
-    class cmd_serve(bzrlib.builtins.cmd_serve):
-        __doc__ = _original_command.__doc__
-
-        takes_options = _original_command.takes_options + [
-            Option('http',
-                help='Run an http (Loggerhead) server to browse code, '
-                    'by default on port %s.' % DEFAULT_PORT)]
-
-        def run(self, *args, **kw):
-            if 'http' in kw:
-                # loggerhead internal code will try to 'import loggerhead', so
-                # let's put it on the path
-                import os.path, sys
-                sys.path.append(os.path.dirname(__file__))
-
-                from bzrlib.transport import get_transport
-                from loggerhead.apps.transport import BranchesFromTransportRoot
-                from loggerhead.config import LoggerheadConfig
-                from paste.httpexceptions import HTTPExceptionHandler
-                from paste.httpserver import serve
-                port = kw.get('port', DEFAULT_PORT)
-                # port might be an int already...
-                if isinstance(port, basestring) and ':' in port:
-                    host, port = port.split(':')
+    HELP = ('Loggerhead, a web-based code viewer and server. (default port: %d)' %
+            (DEFAULT_PORT,))
+
+    def serve_http(transport, host=None, port=None, inet=None):
+        # loggerhead internal code will try to 'import loggerhead', so
+        # let's put it on the path if we can't find it in the existing path
+        try:
+            import loggerhead
+        except ImportError:
+            import os.path, sys
+            sys.path.append(os.path.dirname(__file__))
+
+        from loggerhead.apps.transport import BranchesFromTransportRoot
+        from loggerhead.config import LoggerheadConfig
+        from paste.httpexceptions import HTTPExceptionHandler
+        from paste.httpserver import serve
+        if host is None:
+            host = DEFAULT_HOST
+        if port is None:
+            port = DEFAULT_PORT
+        argv = ['--host', host, '--port', str(port), '--', transport.base]
+        if not transport.is_readonly():
+            argv.insert(0, '--allow-writes')
+        config = LoggerheadConfig(argv)
+        app = BranchesFromTransportRoot(transport, config)
+        app = HTTPExceptionHandler(app)
+        serve(app, host=host, port=port)
+
+    if transport_server_registry is not None:
+        transport_server_registry.register('http', serve_http, help=HELP)
+    else:
+        import bzrlib.builtins
+        from bzrlib.commands import get_cmd_object, register_command
+        from bzrlib.option import Option
+
+        _original_command = get_cmd_object('serve')
+
+        class cmd_serve(bzrlib.builtins.cmd_serve):
+            __doc__ = _original_command.__doc__
+
+            takes_options = _original_command.takes_options + [
+                Option('http', help=HELP)]
+
+            def run(self, *args, **kw):
+                if 'http' in kw:
+                    from bzrlib.transport import get_transport
+                    allow_writes = kw.get('allow_writes', False)
+                    path = kw.get('directory', '.')
+                    port = kw.get('port', DEFAULT_PORT)
+                    # port might be an int already...
+                    if isinstance(port, basestring) and ':' in port:
+                        host, port = port.split(':')
+                    else:
+                        host = DEFAULT_HOST
+                    if allow_writes:
+                        transport = get_transport(path)
+                    else:
+                        transport = get_transport('readonly+' + path)
+                    serve_http(transport, host, port)
                 else:
-                    host = '0.0.0.0'
-                argv = ['--host', host, '--port', str(port)]
-                config = LoggerheadConfig(argv)
-                transport = get_transport('.')
-                app = BranchesFromTransportRoot(transport, config)
-                app = HTTPExceptionHandler(app)
-                serve(app, host=host, port=port)
-            else:
-                super(cmd_serve, self).run(*args, **kw)
+                    super(cmd_serve, self).run(*args, **kw)
 
-    register_command(cmd_serve)
+        register_command(cmd_serve)

=== modified file 'loggerhead/__init__.py'
--- loggerhead/__init__.py	2009-02-10 21:11:43 +0000
+++ loggerhead/__init__.py	2009-05-29 13:39:28 +0000
@@ -1,3 +1,6 @@
+#
+# Copyright (C) 2008, 2009 Canonical Ltd.
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -20,7 +23,7 @@
 import pkg_resources
 
 __version__ = '1.10'
-required_bzrlib = (1, 6)
+required_bzrlib = (1, 13)
 
 pkg_resources.get_distribution('Paste>=1.6')
 try:

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2009-05-13 11:34:09 +0000
+++ loggerhead/apps/branch.py	2009-05-28 08:17:40 +0000
@@ -1,3 +1,19 @@
+# Copyright (C) 2008, 2009 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
 """The WSGI application for serving a Bazaar branch."""
 
 import logging
@@ -113,6 +129,10 @@
         return self.branch.get_config().get_user_option('public_branch')
 
     def app(self, environ, start_response):
+        # Check again if the branch is blocked from being served, this is
+        # mostly for tests. It's already checked in apps/transport.py
+        if self.branch.get_config().get_user_option('http_serve') == 'False':
+            raise httpexceptions.HTTPNotFound()
         self._url_base = environ['SCRIPT_NAME']
         self._static_url_base = environ.get('loggerhead.static.url')
         if self._static_url_base is None:
@@ -123,7 +143,12 @@
             if public_branch is not None:
                 self.served_url = public_branch
             else:
-                self.served_url = self.url([])
+                # Loggerhead only supports serving .bzr/ on local branches, so
+                # we shouldn't suggest something that won't work.
+                if self.branch.base.startswith('file://'):
+                    self.served_url = self.url([])
+                else:
+                    self.served_url = None
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(

=== modified file 'loggerhead/apps/transport.py'
--- loggerhead/apps/transport.py	2009-05-04 20:28:50 +0000
+++ loggerhead/apps/transport.py	2009-06-17 23:10:07 +0000
@@ -1,6 +1,24 @@
+# Copyright (C) 2008, 2009 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
 """Serve branches at urls that mimic a transport's file system layout."""
 
 from bzrlib import branch, errors, lru_cache, urlutils
+from bzrlib.bzrdir import BzrDir
+from bzrlib.transport.http import wsgi
 
 from paste.request import path_info_pop
 from paste import httpexceptions
@@ -63,7 +81,10 @@
                 raise httpexceptions.HTTPNotFound()
             return self.app_for_non_branch(environ)(environ, start_response)
         else:
-            return self.app_for_branch(b)(environ, start_response)
+            if b.get_config().get_user_option('http_serve') == 'False':
+                raise httpexceptions.HTTPNotFound()
+            else:
+                return self.app_for_branch(b)(environ, start_response)
 
 
 class BranchesFromTransportRoot(object):
@@ -71,8 +92,33 @@
     def __init__(self, transport, config):
         self.graph_cache = lru_cache.LRUCache(10)
         self.transport = transport
+        wsgi_app = wsgi.SmartWSGIApp(self.transport)
+        self.smart_server_app = wsgi.RelpathSetter(wsgi_app, '', 'PATH_INFO')
         self._config = config
 
+    def get_local_path(self):
+        """Raise exception if it's not a local path, otherwise return it"""
+
+        # TODO: Use something here that uses the transport API 
+        # rather than relying on the local filesystem API.
+        try:
+            path = urlutils.local_path_from_url(self.transport.base)
+        except errors.InvalidURL:
+            raise httpexceptions.HTTPNotFound()
+        else:
+            return path
+
+    def check_is_a_branch(self, path_info):
+        """Check if it's a branch, and that it's allowed to be shown"""
+        try:
+            bzrdir = BzrDir.open_containing_from_transport(
+                       self.transport.clone(path_info))[0]
+            branch = bzrdir.open_branch()
+            if branch.get_config().get_user_option('http_serve') == 'False':
+                raise httpexceptions.HTTPNotFound()
+        except errors.NotBranchError:
+            return
+
     def __call__(self, environ, start_response):
         environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
         if environ['PATH_INFO'].startswith('/static/'):
@@ -81,8 +127,13 @@
             return static_app(environ, start_response)
         elif environ['PATH_INFO'] == '/favicon.ico':
             return favicon_app(environ, start_response)
+        elif environ['PATH_INFO'].endswith("/.bzr/smart"):
+            self.check_is_a_branch(environ['PATH_INFO'])
+            return self.smart_server_app(environ, start_response)
         elif '/.bzr/' in environ['PATH_INFO']:
-            app = urlparser.make_static(None, self.transport)
+            self.check_is_a_branch(environ['PATH_INFO'])
+            path = self.get_local_path()
+            app = urlparser.make_static(None, path)
             return app(environ, start_response)
         else:
             return BranchesFromTransportServer(

=== modified file 'loggerhead/config.py'
--- loggerhead/config.py	2009-05-17 11:10:38 +0000
+++ loggerhead/config.py	2009-06-03 23:21:52 +0000
@@ -1,8 +1,23 @@
+#
+# Copyright (C) 2008, 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
 '''Configuration tools for Loggerhead.'''
+
 from optparse import OptionParser
 import sys
 import tempfile
 
+from bzrlib import config
 
 _temporary_sql_dir = None
 
@@ -20,6 +35,7 @@
         log_folder=None,
         use_cdn=False,
         sql_dir=None,
+        allow_writes=False,
         )
     parser.add_option("--user-dirs", action="store_true", dest="user_dirs",
                       help="Serve user directories as ~user.")
@@ -32,7 +48,7 @@
                       help="Host Loggerhead should listen on.")
     parser.add_option('--memory-profile', action='store_true',
                       dest='memory_profile',
-                      help='Profile the memory usage using heapy.')
+                      help='Profile the memory usage using Dozer.')
     parser.add_option("--prefix", dest="user_prefix",
                       help="Specify host prefix.")
     parser.add_option("--profile", action="store_true", dest="profile",
@@ -45,10 +61,12 @@
                       type=str, help="The directory to place log files in.")
     parser.add_option("--version", action="store_true", dest="show_version",
                       help="Print the software version and exit")
-    parser.add_option('--use-cdn', action='store_true',
+    parser.add_option("--use-cdn", action="store_true", dest="use_cdn",
                       help="Serve YUI from Yahoo!'s CDN")
-    parser.add_option('--cache-dir', dest='sql_dir',
+    parser.add_option("--cache-dir", dest="sql_dir",
                       help="The directory to place the SQL cache in")
+    parser.add_option('--allow-writes', action='store_true',
+                      help="Allow writing to the Bazaar server.")
     return parser
 
 
@@ -57,7 +75,7 @@
 
     def __init__(self, argv=None):
         if argv is None:
-            sys.argv[1:]
+            argv = sys.argv[1:]
         self._parser = command_line_parser()
         self._options, self._args = self._parser.parse_args(argv)
 
@@ -67,19 +85,27 @@
         self.SQL_DIR = sql_dir
 
     def get_option(self, option):
-        '''Get an option from the options dict.'''
-        return getattr(self._options, option)
+        """Get the value for the config option, either 
+           from ~/.bazaar/bazaar.conf or from the command line.
+           All loggerhead-specific settings start with 'http_'"""
+        global_config = config.GlobalConfig().get_user_option('http_'+option)
+        cmd_config = getattr(self._options, option)
+        if global_config is not None and (
+                cmd_config is None or cmd_config is False):
+            return global_config
+        else:
+            return cmd_config
 
     def get_arg(self, index):
-        '''Get an arg from the arg list.'''
+        """Get an arg from the arg list."""
         return self._args[index]
 
     def print_help(self):
-        '''Wrapper around OptionParser.print_help.'''
+        """Wrapper around OptionParser.print_help."""
         return self._parser.print_help()
 
     @property
     def arg_count(self):
-        '''Return the number of args from the option parser.'''
+        """Return the number of args from the option parser."""
         return len(self._args)
 

=== modified file 'loggerhead/controllers/atom_ui.py'
--- loggerhead/controllers/atom_ui.py	2008-12-05 18:52:44 +0000
+++ loggerhead/controllers/atom_ui.py	2009-06-06 09:09:55 +0000
@@ -35,6 +35,6 @@
         headers['Content-Type'] = 'application/atom+xml'
         return {
             'changes': entries,
-            'updated': entries[0].date.isoformat() + 'Z',
+            'updated': entries[0].utc_date.isoformat(),
             'history': self._history,
         }

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2009-03-17 22:38:57 +0000
+++ loggerhead/controllers/changelog_ui.py	2009-05-29 13:39:28 +0000
@@ -1,4 +1,5 @@
 #
+# Copyright (C) 2008, 2009 Canonical Ltd.
 # Copyright (C) 2006  Robey Pointer <robey@lag.net>
 # Copyright (C) 2006  Goffredo Baroncelli <kreijack@inwind.it>
 #

=== modified file 'loggerhead/controllers/directory_ui.py'
--- loggerhead/controllers/directory_ui.py	2009-05-05 18:37:26 +0000
+++ loggerhead/controllers/directory_ui.py	2009-06-02 13:39:33 +0000
@@ -20,7 +20,7 @@
 import logging
 import stat
 
-from bzrlib import branch
+from bzrlib import branch, errors
 
 from loggerhead import util
 from loggerhead.controllers import TemplatedBranchView
@@ -72,8 +72,13 @@
         for d in listing:
             try:
                 b = branch.Branch.open_from_transport(self.transport.clone(d))
+                if b.get_config().get_user_option('http_serve') == 'False':
+                    continue
             except:
-                if not stat.S_ISDIR(self.transport.stat(d).st_mode):
+                try:
+                    if not stat.S_ISDIR(self.transport.stat(d).st_mode):
+                        continue
+                except errors.NoSuchFile:
                     continue
                 b = None
             dirs.append(DirEntry(d, parity, b))

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2009-03-19 19:04:40 +0000
+++ loggerhead/controllers/revision_ui.py	2009-04-29 19:55:16 +0000
@@ -95,6 +95,18 @@
 
         h.add_branch_nicks(change)
 
+        if '.' in change.revno:
+            # Walk "down" though the merge-sorted graph until we find a
+            # revision with merge depth 0: this is the revision that merged
+            # this one to mainline.
+            ri = self._history._rev_info
+            i = self._history._rev_indices[change.revid]
+            while ri[i][0][2] > 0:
+                i += 1
+            merged_in = ri[i][0][3]
+        else:
+            merged_in = None
+
         # Directory Breadcrumbs
         directory_breadcrumbs = (
             util.directory_breadcrumbs(
@@ -116,6 +128,7 @@
             'filter_file_id': filter_file_id,
             'util': util,
             'history': h,
+            'merged_in': merged_in,
             'navigation': navigation,
             'query': query,
             'remember': remember,

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2009-04-24 10:30:15 +0000
+++ loggerhead/history.py	2009-06-09 03:41:07 +0000
@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2008  Canonical Ltd.
+# Copyright (C) 2008, 2009 Canonical Ltd.
 #                     (Authored by Martin Albisetti <argentina@gmail.com>)
 # Copyright (C) 2006  Robey Pointer <robey@lag.net>
 # Copyright (C) 2006  Goffredo Baroncelli <kreijack@inwind.it>
@@ -31,13 +31,9 @@
 import bisect
 import datetime
 import logging
-import marshal
 import re
 import textwrap
 import threading
-import time
-import urllib
-from StringIO import StringIO
 
 from loggerhead import search
 from loggerhead import util
@@ -112,9 +108,6 @@
     return path
 
 
-# from bzrlib
-
-
 class _RevListToTimestamps(object):
     """This takes a list of revisions, and allows you to bisect by date"""
 
@@ -236,9 +229,6 @@
         parents of this revision.
     :ivar _rev_indices: A dictionary mapping each revision id to the index of
         the information about it in _rev_info.
-    :ivar _full_history: A list of all revision ids in the ancestry of the
-        branch, in merge-sorted order.  This is a bit silly, and shouldn't
-        really be stored on the instance...
     :ivar _revno_revid: A dictionary mapping stringified revnos to revision
         ids.
     """
@@ -271,19 +261,15 @@
             update_missed_caches()
 
         if self._rev_indices is not None:
-            self._full_history = []
             self._revno_revid = {}
             for ((_, revid, _, revno_str, _), _, _) in self._rev_info:
                 self._revno_revid[revno_str] = revid
-                self._full_history.append(revid)
         else:
-            self._full_history = []
             self._revno_revid = {}
             self._rev_indices = {}
             for ((seq, revid, _, revno_str, _), _, _) in self._rev_info:
                 self._rev_indices[revid] = seq
                 self._revno_revid[revno_str] = revid
-                self._full_history.append(revid)
 
     def __init__(self, branch, whole_history_data_cache, file_cache=None,
                  revinfo_disk_cache=None, cache_key=None):
@@ -327,7 +313,7 @@
         revid in revid_list.
         """
         if revid_list is None:
-            revid_list = self._full_history
+            revid_list = [r[0][1] for r in self._rev_info]
         revid_set = set(revid_list)
         revid = start_revid
 
@@ -353,25 +339,17 @@
     def get_short_revision_history_by_fileid(self, file_id):
         # FIXME: would be awesome if we could get, for a folder, the list of
         # revisions where items within that folder changed.i
-        try:
-            # FIXME: Workaround for bzr versions prior to 1.6b3.
-            # Remove me eventually pretty please  :)
-            w = self._branch.repository.weave_store.get_weave(
-                    file_id, self._branch.repository.get_transaction())
-            w_revids = w.versions()
-            revids = [r for r in self._full_history if r in w_revids]
-        except AttributeError:
-            possible_keys = [(file_id, revid) for revid in self._full_history]
-            get_parent_map = self._branch.repository.texts.get_parent_map
-            # We chunk the requests as this works better with GraphIndex.
-            # See _filter_revisions_touching_file_id in bzrlib/log.py
-            # for more information.
-            revids = []
-            chunk_size = 1000
-            for start in xrange(0, len(possible_keys), chunk_size):
-                next_keys = possible_keys[start:start + chunk_size]
-                revids += [k[1] for k in get_parent_map(next_keys)]
-            del possible_keys, next_keys
+        possible_keys = [(file_id, revid) for revid in self._rev_indices]
+        get_parent_map = self._branch.repository.texts.get_parent_map
+        # We chunk the requests as this works better with GraphIndex.
+        # See _filter_revisions_touching_file_id in bzrlib/log.py
+        # for more information.
+        revids = []
+        chunk_size = 1000
+        for start in xrange(0, len(possible_keys), chunk_size):
+            next_keys = possible_keys[start:start + chunk_size]
+            revids += [k[1] for k in get_parent_map(next_keys)]
+        del possible_keys, next_keys
         return revids
 
     def get_revision_history_since(self, revid_list, date):
@@ -671,8 +649,6 @@
         Given a bzrlib Revision, return a processed "change" for use in
         templates.
         """
-        commit_time = datetime.datetime.fromtimestamp(revision.timestamp)
-
         parents = [util.Container(revid=r,
                    revno=self.get_revno(r)) for r in revision.parent_ids]
 
@@ -685,13 +661,15 @@
 
         entry = {
             'revid': revision.revision_id,
-            'date': commit_time,
+            'date': datetime.datetime.fromtimestamp(revision.timestamp),
+            'utc_date': datetime.datetime.utcfromtimestamp(revision.timestamp),
             'authors': authors,
             'branch_nick': revision.properties.get('branch-nick', None),
             'short_comment': short_message,
             'comment': revision.message,
             'comment_clean': [util.html_clean(s) for s in message],
             'parents': revision.parent_ids,
+            'bugs': [bug.split()[0] for bug in revision.properties.get('bugs', '').splitlines()],
         }
         return util.Container(entry)
 

=== modified file 'loggerhead/static/css/diff.css'
--- loggerhead/static/css/diff.css	2009-02-26 20:55:46 +0000
+++ loggerhead/static/css/diff.css	2009-05-30 17:40:05 +0000
@@ -31,7 +31,7 @@
     width:493px;
     margin:0px 0px 5px 0px;
 }
-.information, .committer, .timer, .mfrom, .mto, .revid {
+.information, .committer, .timer, .mfrom, .mto, .revid, .bug {
     clear:both;
     margin:0px;
     padding:0px 0px 2px 18px;
@@ -65,6 +65,14 @@
 .mto {
     background:url(../images/ico_mergeto.gif) no-repeat top left;
 }
+.bug {
+    padding:1px 0px 0px 18px;
+    margin:0px 10px 5px 0px;
+    float:left;
+    clear:none;
+    display:inline;
+    background:url(../images/ico_bug.png) no-repeat top left;
+}
 /**/
 #loggerheadCont p.expand, #loggerheadCont p.expand a, #loggerheadCont p.expand a:link, #loggerheadCont p.expand a:visited, #loggerheadCont p.expand a:hover {
     color:#0f6b97;

=== modified file 'loggerhead/static/css/files.css'
--- loggerhead/static/css/files.css	2009-01-23 20:53:40 +0000
+++ loggerhead/static/css/files.css	2009-05-28 22:48:45 +0000
@@ -1,5 +1,5 @@
 /*Info Box*/
-.information, .committer, .timer, .mfrom, .mto, .revid {
+.information, .committer, .timer, .mfrom, .mto, .revid, .bug {
 	float:left;
 	margin:0px 10px 5px 0px;
 	padding:0px 0px 2px 18px;
@@ -39,3 +39,6 @@
     width:100%;
     position:relative;
 }
+.bug {
+	background:url(../images/ico_bug.png) no-repeat top left;
+}

=== added file 'loggerhead/static/images/ico_bug.png'
Binary files loggerhead/static/images/ico_bug.png and loggerhead/static/images/ico_bug.png differ
=== modified file 'loggerhead/templatefunctions.py'
--- loggerhead/templatefunctions.py	2009-03-19 00:18:46 +0000
+++ loggerhead/templatefunctions.py	2009-04-28 03:01:54 +0000
@@ -57,12 +57,12 @@
 
 
 @templatefunc
-def revisioninfo(url, branch, entry, file_changes=None, currently_showing=None):
+def revisioninfo(url, branch, entry, file_changes=None, currently_showing=None, merged_in=None):
     from loggerhead import util
     return _pt('revisioninfo').expand(
         url=url, change=entry, branch=branch, util=util,
         file_changes=file_changes, currently_showing=currently_showing,
-        **templatefunctions)
+        merged_in=merged_in, **templatefunctions)
 
 
 @templatefunc

=== modified file 'loggerhead/templates/atom.pt'
--- loggerhead/templates/atom.pt	2009-03-06 09:57:54 +0000
+++ loggerhead/templates/atom.pt	2009-06-06 12:23:11 +0000
@@ -3,17 +3,19 @@
   <title>
     bazaar changes for <tal:branch-name content="branch/friendly_name">branch name</tal:branch-name>
   </title>
-  <updated tal:content="updated">${updated}</updated>
+  <updated tal:content="string:${updated}Z">${updated}</updated>
   <id tal:content="python:branch.url(['/atom'])">url</id>
-  <link rel="self" tal:attributes="href python:branch.url([''])" />
+  <link rel="self" type="application/atom+xml" tal:attributes="href python:branch.url(['/atom'])" />
+  <link rel="alternate" type="text/html" tal:attributes="href python:branch.url(['/changes'])" />
 
   <entry tal:repeat="entry changes">
-    <title tal:content="string: ${entry/revno}: ${entry/short_comment}">
+    <title tal:content="string:${entry/revno}: ${entry/short_comment}">
       ${entry/revno}: ${entry/short_comment}
     </title>
-    <updated tal:content="entry/date/isoformat">
-    Z
+    <updated tal:content="string:${entry/utc_date/isoformat}Z">
+      updated
     </updated>
+    <!-- TODO: The feed validator will generate warnings because this is URL-encoded -->
     <id tal:content="python:branch.url(['/revision', entry.revno])">
       ID
     </id>
@@ -25,7 +27,7 @@
     <content type="text" tal:content="entry/comment">
       comment
     </content>
-    <link rel="alternate"
+    <link rel="alternate" type="text/html"
           tal:attributes="href python:branch.url(['/revision', entry.revno])" />
   </entry>
 </feed>

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2009-03-31 21:15:48 +0000
+++ loggerhead/templates/revision.pt	2009-04-28 03:01:54 +0000
@@ -96,7 +96,7 @@
           </ul>
         </tal:we-are-comparing>
 
-        <tal:revision-info replace="structure python:revisioninfo(url, branch, change, file_changes, specific_path)" />
+        <tal:revision-info replace="structure python:revisioninfo(url, branch, change, file_changes, specific_path, merged_in)" />
         <tal:specific-path condition="not:specific_path">
           <p class="expand show_if_js" id="expand_all"><a href="#">
               <img tal:attributes="src python:branch.static_url('/static/images/treeCollapsed.png')"

=== modified file 'loggerhead/templates/revisioninfo.pt'
--- loggerhead/templates/revisioninfo.pt	2009-03-19 00:18:46 +0000
+++ loggerhead/templates/revisioninfo.pt	2009-05-30 17:40:05 +0000
@@ -12,7 +12,16 @@
                                href python:url(['/revision', merge_point.revno], clear=1)"
                tal:content="python:revno_with_nick(merge_point)"></a>
         </div>
+        <div class="mto" tal:condition="merged_in">
+          This revision was merged to the branch mainline in
+          <a tal:attributes="href python:url(['/revision', merged_in], clear=1)">revision
+            <tal:b content="merged_in"/></a>.
+        </div>
         <div class="revid" tal:content="string:Revision ID: ${change/revid}"></div>
+        <div class="clear"><!-- --></div>
+        <div class="bug" tal:repeat="bug change/bugs">
+            <a tal:attributes="href bug" tal:content="bug"></a>
+        </div>
         <div class="information" tal:content="structure python:util.fixed_width(change.comment)"></div>
     </div>
     <ul id="list-files" tal:condition="file_changes">

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2009-02-18 10:40:22 +0000
+++ loggerhead/tests/test_simple.py	2009-06-08 23:02:49 +0000
@@ -1,8 +1,26 @@
+# Copyright (C) 2008, 2009 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
 import cgi
 import logging
 
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.util.configobj.configobj import ConfigObj
+from bzrlib import config
 
 from loggerhead.apps.branch import BranchWSGIApp
 from paste.fixture import TestApp
@@ -103,6 +121,7 @@
 
 
 class TestEmptyBranch(BasicTests):
+    """Test that an empty branch doesn't break"""
 
     def setUp(self):
         BasicTests.setUp(self)
@@ -118,3 +137,37 @@
         res = app.get('/files')
         res.mustcontain('No revisions!')
 
+
+class TestHiddenBranch(BasicTests):
+    """
+    Test that hidden branches aren't shown
+    FIXME: not tested that it doesn't show up on listings
+    """
+
+    def setUp(self):
+        BasicTests.setUp(self)
+        self.createBranch()
+        locations = config.locations_config_filename()
+        config.ensure_config_dir_exists()
+        open(locations, 'wb').write('[%s]\nhttp_serve = False'
+                                    % (self.tree.branch.base,))
+
+    def test_no_access(self):
+        app = self.setUpLoggerhead()
+        res = app.get('/changes', status=404)
+
+
+#class TestGlobalConfig(BasicTests):
+#    """
+#    Test that global config settings are respected
+#    """
+
+#    def setUp(self):
+#        BasicTests.setUp(self)
+#        self.createBranch()
+#        config.GlobalConfig().set_user_option('http_version', 'True')
+
+#    def test_setting_respected(self):
+        #FIXME: Figure out how to test this properly
+#        app = self.setUpLoggerhead()
+#        res = app.get('/changes', status=200)

=== modified file 'loggerhead/wholehistory.py'
--- loggerhead/wholehistory.py	2009-04-24 10:30:15 +0000
+++ loggerhead/wholehistory.py	2009-05-29 13:39:28 +0000
@@ -1,4 +1,5 @@
-# Cache the whole history data needed by loggerhead about a branch.
+#
+# Copyright (C) 2008, 2009 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,6 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
+"""Cache the whole history data needed by loggerhead about a branch."""
 
 import logging
 import time
@@ -53,7 +55,6 @@
          graph.iter_ancestry([last_revid]) if value is not None))
 
     _revision_graph = _strip_NULL_ghosts(parent_map)
-    _full_history = []
 
     _rev_info = []
     _rev_indices = {}
@@ -66,7 +67,6 @@
 
     for info in _merge_sort:
         seq, revid, merge_depth, revno, end_of_merge = info
-        _full_history.append(revid)
         revno_str = '.'.join(str(n) for n in revno)
         parents = _revision_graph[revid]
         _rev_indices[revid] = len(_rev_info)

=== modified file 'serve-branches'
--- serve-branches	2009-05-04 20:21:08 +0000
+++ serve-branches	2009-06-05 01:59:11 +0000
@@ -1,4 +1,7 @@
 #!/usr/bin/env python
+#
+# Copyright (C) 2008, 2009 Canonical Ltd
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -51,7 +54,10 @@
 
     load_plugins()
 
-    transport = get_transport(path)
+    if config.get_option('allow_writes'):
+        transport = get_transport(path)
+    else:
+        transport = get_transport('readonly+' + path)
 
     if config.get_option('trunk_dir') and not config.get_option('user_dirs'):
         print "--trunk-dir is only valid with --user-dirs"
@@ -89,7 +95,6 @@
 
     # setup_logging() #end
 
-    app = TransLogger(app, logger=logger)
     if config.get_option('profile'):
         from loggerhead.middleware.profile import LSProfMiddleware
         app = LSProfMiddleware(app)
@@ -124,6 +129,7 @@
 
     app = HTTPExceptionHandler(app)
     app = ErrorHandlerApp(app)
+    app = TransLogger(app, logger=logger)
 
     if not config.get_option('user_port'):
         port = '8080'

=== modified file 'setup.py' (properties changed: -x to +x)
--- setup.py	2008-11-20 16:55:47 +0000
+++ setup.py	2009-06-02 22:10:00 +0000
@@ -34,7 +34,9 @@
     packages = ["loggerhead",
                 "loggerhead/apps",
                 "loggerhead/controllers",
-                "loggerhead/templates"],
+                "loggerhead/templates",
+                "bzrlib.plugins.loggerhead"],
+    package_dir={'bzrlib.plugins.loggerhead':'.'},
     package_data = {"loggerhead": ["templates/*.pt",
                                    "static/css/*.css",
                                    "static/javascript/*.js",

=== modified file 'start-loggerhead'
--- start-loggerhead	2009-02-19 01:24:20 +0000
+++ start-loggerhead	2009-06-08 23:02:49 +0000
@@ -13,7 +13,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""A script for starting the loggerhead process."""
+"""
+WARNING! This script has been deprecated and will go away soon!
+A script for starting the loggerhead process.
+"""
 
 
 import logging
@@ -37,6 +40,13 @@
 
 
 def main():
+    sys.stderr.write('\n\n')
+    sys.stderr.write('WARNING!!! This script has been deprecated by '
+                     'serve-branches, and will be removed in the next '
+                     'release. Please migrate to serve-branches and report '
+                     'any missing features.\n')
+    sys.stderr.write('\n\n')
+
     home = os.path.realpath(os.path.dirname(__file__))
     default_pidfile = os.path.join(home, 'loggerhead.pid')
     default_configfile = os.path.join(home, 'loggerhead.conf')

